package com.persagy.cache;

import java.util.Objects;
import java.util.Optional;

import javax.annotation.Resource;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.persagy.constant.RedisConstant;
import com.persagy.entity.AlarmConditionState;
import com.persagy.entity.ZktAlarmRecordDO;
import com.persagy.repository.AlarmRecordRepository;
import lombok.extern.slf4j.Slf4j;

@Component
@Slf4j
public class AlarmRedisCache implements RedisCache {

	@Resource(name = "emsRedisTemplate")
	private RedisTemplate<String, String> emsRedisTemplate;

	@Autowired
	AlarmRecordRepository alarmRecordRepository;

	public AlarmConditionState getAlarmDefineState(String defineId) {
		AlarmConditionState alarmConditionState = null;
		String str = emsRedisTemplate.opsForHash().get(RedisConstant.DEFINE_ID, defineId).toString();
		if (StringUtils.isNotBlank(str)) {
			try {
				JSONObject object = JSONObject.parseObject(str);
				alarmConditionState = JSON.toJavaObject(object, AlarmConditionState.class);
			} catch (Exception e) {
				log.error(" alarmDefineState json 转换 AlarmConditionState 失败", e);
			}
		}
		try {
			if (Objects.isNull(alarmConditionState)) {
				// 默认正常报警状态
				Optional<ZktAlarmRecordDO> recordOptional = alarmRecordRepository.findById(defineId);
				if (recordOptional.isPresent()) {
					ZktAlarmRecordDO alarmRecordDO = recordOptional.get();
					// 数据库报警状态：1-未处理
					if ("1".equals(alarmRecordDO.getState())) {
						alarmConditionState = new AlarmConditionState(defineId);
						alarmConditionState.setState(AlarmConditionState.State.NOT_DEAL.getType());
						alarmConditionState.setAlarmStartTime(alarmRecordDO.getAlarmTime());
					}
				}
			}
			log.debug("alarmDefineState: [{}], （ 0-正常 1-报警）", alarmConditionState.getState());
		} catch (Exception e) {
			log.warn("从redis获取报警定义状态失败" + e);
		}
		return alarmConditionState;
	}

	public void setAlarmState(String defineId, AlarmConditionState alarmConditionState) {
		emsRedisTemplate.opsForHash().put(RedisConstant.DEFINE_ID, defineId, JSON.toJSONString(alarmConditionState));
	}
}
