package com.persagy.mq;

import org.springframework.amqp.core.Binding;
import org.springframework.amqp.core.BindingBuilder;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.core.Queue;
import org.springframework.amqp.core.TopicExchange;
import org.springframework.amqp.rabbit.annotation.RabbitHandler;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import com.persagy.entity.DmpMessage;
import com.rabbitmq.client.Channel;

import lombok.extern.slf4j.Slf4j;

/**
 * @description:报警定义消息通知
 * @author:LuoGuangyi
 * @company:PersagyTechnologyCo.,Ltd
 * @since:2020/10/20 002016:30
 * @version:V1.0
 **/
@Slf4j
@Configuration
public class RabbitConfig {

	@Autowired
	public RabbitConfig(RabbitTemplate rabbitTemplate) {
		rabbitTemplate.convertAndSend(exchange, alarmMsgRoutingKey, "测试一条数据");
	}

	/**
	 * 交换机
	 */
	public static final String exchange = "exchange-alarm-service";

	/**
	 * 报警消息路由键
	 */
	public static final String alarmMsgRoutingKey = "alarm-msg-routing-key";

	/**
	 * 报警条件队列
	 */
	private final String alarmMsgQueue = "alarm-msg-queue";

	@Bean
	public Queue alarmMsgQueue() {
		return new Queue(alarmMsgQueue, true);
	}

	@Bean
	public TopicExchange alarmServiceExchange() {
		return new TopicExchange(exchange);
	}

	@Bean
	public Binding conditionBinding() {
		return BindingBuilder.bind(alarmMsgQueue()).to(alarmServiceExchange()).with(alarmMsgRoutingKey);
	}

	@RabbitHandler
	@RabbitListener(queues = { alarmMsgQueue })
	public void dealMsg(String msg) throws Exception {
		System.out.println(msg);
	}

}
